package org.mykarte.service.retrieve;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.mykarte.service.retrieve.impl.AttachmentFactory;
import org.mykarte.service.retrieve.impl.IAttachment;
import org.mykarte.util.DateUtils;
import org.mykarte.ws.attachment.proxy.AttachmentServiceHttpSoap11EndpointClient;


/**
 * RetrieveService NX
 */
public class RetrieveService {
  // Logging (log4j)
  private Log log = LogFactory.getLog(this.getClass());

  private IAttachment attachment = null;

  private String endpoint = "";
  private String facilityId = "";
  private String patientId = "";
  private String encryptedId = "";
  private String type = "";
  private String attribute = "";
  private String serviceId = "";

  private boolean auth = false;

  /**
   * F؂B
   * 
   * @param args 
   * @return trueFOK, falseFNG
   */
  public boolean authenticate(String[] args) {
    IAttachment attachment = getAttachment();

    auth = attachment.authenticate(args);

    return auth;
  }

  /**
   * R[obNݒ肷B
   * 
   * @param info R[obN
   */
  public void setCallbackInfo(String[] info) throws Exception {
    checkAuth();

    // Gh|Cg
    endpoint = info[0];

    // {ID
    facilityId = info[1];

    // ÍID
    encryptedId = info[2];

    // T[rXID
    serviceId = info[3];
  }

  /**
   * ID擾B
   * 
   * @return ID
   */
  public String getPatient() throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    return attachment.getPatient();
  }

  /**
   * IDݒ肷B
   * 
   * @param patientId ID
   */
  public void setPatient(String patientId) throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    this.patientId = patientId;

    attachment.setPatient(patientId);
  }

  /**
   * type, attribute ݒ肷B
   * 
   * @param props type, attribute
   */
  public void setProperties(String[] props) throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    // ^Cv
    type = props[0];

    // 
    attribute = props[1];

    attachment.setProperties(props);
  }

  /**
   * t擾B
   * 
   * @param startDate Jn
   * @param endDate I
   * @param diff 
   * @return t
   */
  public String[] getDates(String startDate, String endDate, 
                           boolean diff) throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    Calendar sd = Calendar.getInstance();
    sd.setTime(DateUtils.parse(startDate));

    Calendar ed = Calendar.getInstance();
    ed.setTime(DateUtils.parse(endDate));

    Calendar[] dates = attachment.getDates(sd, ed, diff);

    String[] results = new String[dates.length];

    for (int i = 0; i < dates.length; i++) {
      results[i] = 
          DateUtils.format(dates[i].getTime(), DateUtils.ISO_DATE_FORMAT);
    }

    return results;
  }

  /**
   * w肵t̃f[^𕶎ƂĎ擾B
   * 
   * @param date t
   * @return 
   */
  public String getContent(String date) throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    Calendar d = Calendar.getInstance();
    d.setTime(DateUtils.parse(date));

    return attachment.getContent(d);
  }

  /**
   * w肵t̃f[^t@CƂĎ擾B
   * 
   * @param date t
   * @return 
   */
  public String getFiles(String date) throws Exception {
    checkAuth();

    IAttachment attachment = getAttachment();

    //------------------------//
    //   AttachmentT[rX   //
    //------------------------//    
    AttachmentServiceHttpSoap11EndpointClient attachmentService = 
      new AttachmentServiceHttpSoap11EndpointClient();

    attachmentService.setMaintainSession(true);

    // Gh|Cg
    attachmentService.setEndpoint(endpoint);

    if (!attachmentService.authenticate(new String[] { facilityId, 
                                                       encryptedId })) {
      throw new Exception("AttachmentService̔F؂Ɏs܂B");
    }

    // T[rXID
    attachmentService.setServiceId(serviceId);

    // ID
    attachmentService.setPatientId(patientId);

    // ^Cv
    attachmentService.setType(type);

    // 
    attachmentService.setAttribute(attribute);

    // t
    Calendar d = Calendar.getInstance();
    d.setTime(DateUtils.parse(date));
    attachmentService.setDate(d);

    Map<String, String> files = attachment.getFiles(d);
    boolean[] results = new boolean[files.size()];

    int i = 0;
    for (Map.Entry<String, String> entry: files.entrySet()) {
      // gq
      String extension = getExtension(entry.getKey());

      if (StringUtils.isBlank(extension)) {
        throw new Exception("t@C̊gq܂B");
      }

      attachmentService.setExtension(extension);

      // DescriptionID
      String descriptionId = entry.getValue();

      if (StringUtils.isEmpty(descriptionId)) {
        attachmentService.setDescriptionId("");
      } else {
        attachmentService.setDescriptionId(descriptionId);
      }

      try {
        // t@Cf[^
        BufferedInputStream bin = 
          new BufferedInputStream(new FileInputStream(entry.getKey()));
        byte[] fileData = new byte[bin.available()];
        bin.read(fileData);
        bin.close();

        results[i] = attachmentService.store(i, fileData);
      } catch (Exception ex) {
        ex.printStackTrace();
      }

      i++;
    }

    boolean result;
    boolean[] temp = new boolean[results.length];

    Arrays.fill(temp, true);
    result = Arrays.equals(results, temp);

    if (result) {
      // S
      return "Success";
    }

    Arrays.fill(temp, false);
    result = Arrays.equals(results, temp);

    if (result) {
      // Ss
      return "Failure";
    }

    // ꕔs
    return "Warning";
  }

  /**
   * IAttachment 擾B
   * 
   * @return IAttachment
   */
  private IAttachment getAttachment() {
    if (attachment == null) {
      attachment = AttachmentFactory.create();
    }

    return attachment;
  }

  /**
   * F؂Ă邩mFB
   */
  private void checkAuth() throws Exception {
    if (!auth) {
      throw new Exception("F؂Ă܂B");
    }
  }

  /**
   * t@C̊gq擾B
   * 
   * @param path t@CpX
   * @return gq
   */
  private String getExtension(String path) {
    String fileName = new File(path).getName();

    int index = fileName.lastIndexOf(".");
    String extension = fileName.substring(index + 1).toLowerCase();

    return extension;
  }
}
